# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Retrieve additional parameters from the StarHorse sample
"""

from astropy.table import vstack, join
import pyvo
import requests


def step2(r_edr3, starhorse_block, verbose, debug):
    """Retrieve additional parameters from the StarHorse sample

    Parameters
    ----------
    r_edr3 : astropy Table
        Table containing the initial EDR3 query result.
    starhorse_block : int
        Integer number indicating the maximum number of stars whose
        parameters are retrieved in each single query to Gaia@AIP.
        This number should not be too large to avoid an out-of-time
        error (a typical useful value is 100).
    verbose : bool
        If True, display additional information.
    debug : bool
        If True, display the intermediate tables retrieved in the
        different queries to Gaia@AIP.

    Returns
    -------
    r_edr3 : astropy Table
        Updated table containing the additional StarHorse parameters.

    """
    param_starhorse = [
        'dr3_source_id', 'sh_gaiaflag', 'sh_outflag',
        'dist05', 'dist16', 'dist50', 'dist84', 'dist95',
        'av05', 'av16', 'av50', 'av84', 'av95',
        'teff16', 'teff50', 'teff84',
        'logg16', 'logg50', 'logg84',
        'met16', 'met50', 'met84',
        'mass16', 'mass50', 'mass84',
        'xgal', 'ygal', 'zgal', 'rgal',
        'ruwe', 'angular_distance', 'magnitude_difference',
        'proper_motion_propagation', 'dup_max_number'
    ]
    print('<STEP2> Retrieving StarHorse data from Gaia@AIP... (please wait)')
    print(f'        pyvo version {pyvo.__version__}')
    print(f'        TAP service GAIA@AIP')
    nstars_per_block = starhorse_block
    nstars = len(r_edr3)
    nblocks = int(nstars / nstars_per_block)
    r_starhorse = None
    if nstars - nblocks * nstars_per_block > 0:
        nblocks += 1
    for iblock in range(nblocks):
        irow1 = iblock * nstars_per_block
        irow2 = min(irow1 + nstars_per_block, nstars)
        print(f'        Starting query #{iblock + 1} of {nblocks}...')
        dumstr = ','.join([str(item) for item in r_edr3[irow1:irow2]['source_id']])
        query = f"""
        SELECT {','.join(param_starhorse)}
        FROM gaiadr2_contrib.starhorse
        WHERE dr3_source_id IN ({dumstr})
        """
        tap_session = requests.Session()
        tap_session.headers['Authorization'] = "kkk"
        tap_service = pyvo.dal.TAPService('https://gaia.aip.de/tap', session=tap_session)
        tap_result = tap_service.run_sync(query)
        if debug:
            print(tap_result.to_table())
        if iblock == 0:
            r_starhorse = tap_result.to_table()
        else:
            r_starhorse = vstack([r_starhorse, tap_result.to_table()],
                                 join_type='exact', metadata_conflicts='silent')

    nstars_starhorse = len(r_starhorse)
    if verbose:
        if nstars_starhorse > 0:
            r_starhorse.pprint(max_width=1000)

    # join tables
    print(f'        --> {nstars_starhorse} stars found in StarHorse')
    print('        Joining EDR3 and StarHorse queries...')
    r_starhorse.rename_column('dr3_source_id', 'source_id')
    r_edr3 = join(r_edr3, r_starhorse, keys='source_id', join_type='outer')
    r_edr3.sort('ra')
    if verbose:
        r_edr3.pprint(max_width=1000)

    return r_edr3



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
DR2 query to identify variable stars
"""

import sys

from astroquery.gaia import Gaia


def step4(ra_center, dec_center, search_radius, g_limit, verbose):
    """Perform DR2 query

    Parameters
    ----------
    ra_center : float
        Right ascension (decimal degree) corresponding to the center
        of the field of view.
    dec_center : float
        Declination (decimal degree) corresponding to the center
        of the field of view.
    search_radius : float
        Radius (decimal degrees) of the field of view.
    g_limit : float
        Limiting Gaia G magnitude.
    verbose : bool
        If True, display additional information.

    Returns
    -------
    r_dr2 : astropy Table
        Table containing the query result.
    nvariables : int
        Number of variable stars found.
    mask_var : numpy array
        Array indicating whether the star is variable or not.

    """
    query = f"""
    SELECT source_id, ra, dec, phot_g_mean_mag, phot_variable_flag

    FROM gaiadr2.gaia_source
    WHERE  1=CONTAINS(
      POINT('ICRS', {ra_center}, {dec_center}), 
      CIRCLE('ICRS',ra, dec, {search_radius}))
    AND phot_g_mean_mag < {g_limit}
    """
    sys.stdout.write('<STEP4> Looking for variable stars in Gaia DR2... (please wait)\n  ')
    sys.stdout.flush()
    job = Gaia.launch_job_async(query)
    r_dr2 = job.get_results()
    nstars_dr2 = len(r_dr2)
    if nstars_dr2 == 0:
        nvariables = 0
        mask_var = None
    else:
        if isinstance(r_dr2['phot_variable_flag'][0], bytes):
            mask_var = r_dr2['phot_variable_flag'] == b'VARIABLE'
        elif isinstance(r_dr2['phot_variable_flag'][0], str):
            mask_var = r_dr2['phot_variable_flag'] == 'VARIABLE'
        else:
            raise SystemExit('Unexpected type of data in column phot_variable_flag')
        nvariables = sum(mask_var)
        print(f'        --> {nstars_dr2} stars in DR2, ({nvariables} initial variables)')
    if nvariables > 0:
        if verbose:
            r_dr2[mask_var].pprint(max_width=1000)

    return r_dr2, nvariables, mask_var



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
RGB predictions of Gaia EDR3 stars

This code is hosted at https://github.com/guaix-ucm/rgblues
Authors: Nicolás Cardiel <cardiel@ucm.es>
         Sergio Pascual <sergiopr@fis.ucm.es>
         Rafael González <rafael08@ucm.es>

matplotlib style
"""

mpl_style = {
    'font.size': 17.0,
    'font.family': 'serif',
    'axes.linewidth': 2
}


# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Perform EDR3 query
"""

import sys

from astropy import units as u
from astropy.table import Column
from astroquery.gaia import Gaia
import numpy as np


def step1(ra_center, dec_center, search_radius, g_limit, verbose):
    """Perform EDR3 query

    Parameters
    ----------
    ra_center : float
        Right ascension (decimal degree) corresponding to the center
        of the field of view.
    dec_center : float
        Declination (decimal degree) corresponding to the center
        of the field of view.
    search_radius : float
        Radius (decimal degrees) of the field of view.
    g_limit : float
        Limiting Gaia G magnitude.
    verbose : bool
        If True, display additional information.

    Returns
    -------
    r_edr3 : astropy Table
        Table containing the query result.
    nstars : int
        Number of stars in 'r_edr3'.
    nstars_colorcut: int
        Number of stars outside the colour cut
        -0.5 < G_BP - G_RP < 2.0 mag.

    """
    query = f"""
    SELECT source_id, ra, dec,
    phot_g_mean_mag, phot_bp_mean_mag, phot_rp_mean_mag

    FROM gaiaedr3.gaia_source
    WHERE 1=CONTAINS(
      POINT('ICRS', {ra_center}, {dec_center}), 
      CIRCLE('ICRS',ra, dec, {search_radius}))
    AND phot_g_mean_mag IS NOT NULL 
    AND phot_bp_mean_mag IS NOT NULL 
    AND phot_rp_mean_mag IS NOT NULL
    AND phot_g_mean_mag < {g_limit}

    ORDER BY ra
    """
    sys.stdout.write('<STEP1> Starting cone search in Gaia EDR3... (please wait)\n  ')
    sys.stdout.flush()
    job = Gaia.launch_job_async(query)
    r_edr3 = job.get_results()
    # compute G_BP - G_RP colour
    r_edr3.add_column(
        Column(r_edr3['phot_bp_mean_mag'] - r_edr3['phot_rp_mean_mag'],
               name='bp_rp', unit=u.mag)
    )
    # colour cut in BP-RP
    mask_colour = np.logical_or((r_edr3['bp_rp'] <= -0.5), (r_edr3['bp_rp'] >= 2.0))
    r_edr3_colorcut = r_edr3[mask_colour]
    nstars = len(r_edr3)
    print(f'        --> {nstars} stars found')
    nstars_colorcut = len(r_edr3_colorcut)
    print(f'        --> {nstars_colorcut} stars outside -0.5 < G_BP-G_RP < 2.0')
    if nstars == 0:
        raise SystemExit('ERROR: no stars found. Change search parameters!')
    if verbose:
        r_edr3.pprint(max_width=1000)

    return r_edr3, nstars, nstars_colorcut



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Compute intersection of EDR3 query with the 15M star sample
"""

import sys

import numpy as np


def step3(r_edr3, edr3_source_id_15M_allsky, debug):
    """Compute intersection of EDR3 query with the 15M star sample

    Parameters
    ----------
    r_edr3 : astropy Table
        Table containing the initial EDR3 query result.
    edr3_source_id_15M_allsky : numpy array
        Array with source_id values for the 15M star sample.
    debug : bool
        If True, display debugging information.

    Returns
    -------
    intersection : numpy array
        Array with the source_id values of the intersection between
        the EDR3 query and the 15M star sample.

    """
    sys.stdout.write('<STEP3> Cross-matching EDR3 with 15M subsample... (please wait)')
    sys.stdout.flush()
    set1 = set(np.array(r_edr3['source_id']))
    set2 = set(edr3_source_id_15M_allsky)
    intersection = set2.intersection(set1)
    print(f'\n        --> {len(intersection)} stars in common with 15M sample')
    if debug:
        print(len(set1), len(set2), len(intersection))

    return intersection



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Generate output CSV files
"""

import glob
import os
import sys

from astropy.table import Column
import numpy as np

OUTTYPES = ['edr3', '15m', 'var']


def step7(r_edr3, basename, starhorse_block, nvariables, r_cross_var,
          intersection, table15M, verbose, debug):
    """Generate output CSV files

    Parameters
    ----------
    r_edr3 : astropy Table
        Table containing the EDR3 query.
    basename : str
        Base name for output files.
    starhorse_block : int
        Integer number indicating the maximum number of stars whose
        parameters are retrieved in each single query to Gaia@AIP.
        This number should not be too large to avoid an out-of-time
        error (a typical useful value is 100).
    nvariables : int
        Final number of variable stars in EDR3 query.
    r_cross_var : astropy Table
        Table with the cross-match of the DR2 and EDR3 queries.
    intersection : numpy array
        Array with the source_id values of the intersection between
        the EDR3 query and the 15M star sample.
    table15M : astropy FITS binary table
        Table containing parameters for the stars in the 15M sample.
    verbose : bool
        If True, display additional information.
    debug : bool
        If True, display debugging information.

    """
    sys.stdout.write('<STEP7> Saving output CSV files...')
    sys.stdout.flush()
    r_edr3.add_column(Column(np.zeros(len(r_edr3)), name='number_csv', dtype=int))
    for item in OUTTYPES:
        r_edr3.add_column(Column(np.zeros(len(r_edr3)), name=f'number_{item}', dtype=int))
    outlist = [f'./{basename}_{ftype}.csv' for ftype in OUTTYPES]
    filelist = glob.glob('./*.csv')
    # remove previous versions of the output files (if present)
    for file in outlist:
        if file in filelist:
            try:
                os.remove(file)
            except OSError:
                print(f'ERROR: while deleting existing file {file}')
    # columns to be saved (use a list to guarantee the same order)
    outcolumns_list = ['source_id', 'ra', 'dec', 'b_rgb', 'g_rgb', 'r_rgb', 'g_br_rgb',
                       'phot_g_mean_mag', 'phot_bp_mean_mag', 'phot_rp_mean_mag']
    # define column format with a dictionary
    outcolumns = {'source_id': '19d',
                  'ra': '14.9f',
                  'dec': '14.9f',
                  'b_rgb': '6.2f',
                  'g_rgb': '6.2f',
                  'r_rgb': '6.2f',
                  'g_br_rgb': '6.2f',
                  'phot_g_mean_mag': '8.4f',
                  'phot_bp_mean_mag': '8.4f',
                  'phot_rp_mean_mag': '8.4f'
                  }
    if set(outcolumns_list) != set(outcolumns.keys()):
        raise SystemExit('ERROR: check outcolumns_list and outcolumns')
    csv_header_ini = 'number,' + ','.join(outcolumns_list)
    flist = []
    for ftype in OUTTYPES:
        f = open(f'{basename}_{ftype}.csv', 'wt')
        flist.append(f)
        if (starhorse_block > 0) and (ftype in ['edr3', '15m']):
            if debug and (ftype == '15m'):
                csv_header = csv_header_ini + \
                             ',av50,met50,dist50,b_rgb_bis,g_rgb_bis,r_rgb_bis,g_br_rgb_bis,' \
                             'phot_g_mean_mag_bis,phot_bp_mean_mag_bis,phot_rp_mean_mag_bis,' \
                             'av50_bis,met50_bis,dist50_bis'
            else:
                csv_header = csv_header_ini + ',av50,met50,dist50'
        else:
            csv_header = csv_header_ini
        f.write(csv_header + '\n')
    # save each star in its corresponding output file
    krow = np.ones(len(OUTTYPES), dtype=int)
    for irow, row in enumerate(r_edr3):
        cout = []
        for item in outcolumns_list:
            cout.append(eval("f'{row[item]:" + f'{outcolumns[item]}' + "}'"))
        iout = 0
        if nvariables > 0:
            if row['source_id'] in r_cross_var['dr3_source_id']:
                iout = 2
        if iout == 0:
            if starhorse_block > 0:
                for item in ['av50', 'met50', 'dist50']:
                    value = row[item]
                    if isinstance(value, float):
                        pass
                    else:
                        value = 99.999
                    cout.append(f'{value:7.3f}')
            if row['source_id'] in intersection:
                iout = 1
                if debug:
                    iloc = np.argwhere(table15M['source_id'] == row['source_id'])[0][0]
                    cout.append(f"{table15M['B_rgb'][iloc]:6.2f}")
                    cout.append(f"{table15M['G_rgb'][iloc]:6.2f}")
                    cout.append(f"{table15M['R_rgb'][iloc]:6.2f}")
                    cout.append(f"{table15M['G_BR_rgb'][iloc]:6.2f}")
                    cout.append(f"{table15M['G_gaia'][iloc]:8.4f}")
                    cout.append(f"{table15M['BP_gaia'][iloc]:8.4f}")
                    cout.append(f"{table15M['RP_gaia'][iloc]:8.4f}")
                    cout.append(f"{table15M['av50'][iloc]:7.3f}")
                    cout.append(f"{table15M['met50'][iloc]:7.3f}")
                    cout.append(f"{table15M['dist50'][iloc]:7.3f}")
        flist[iout].write(f'{krow[iout]:6d}, ' + ','.join(cout) + '\n')
        r_edr3[irow]['number_csv'] = iout
        r_edr3[irow][f'number_{OUTTYPES[iout]}'] = krow[iout]
        krow[iout] += 1
    for f in flist:
        f.close()
    print('OK')

    if verbose:
        print(r_edr3)



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Test generation of pleiades cone search used in the paper
"""

import pooch

from rgblues.__main__ import exec_rgblues
from rgblues.core.step7 import OUTTYPES


class Args:
    ra_center = 56.66
    dec_center = 24.10
    search_radius = 1.0
    g_limit = 12.0
    basename = 'rgblues'
    brightlimit = 8.0
    symbsize = 1.0
    nonumbers = False
    noplot = True            # do not generate PDF file
    nocolor = False
    starhorse_block = 0
    verbose = False
    debug = False


auxhash = {
    'edr3': "md5:5b443fbc2863ff8c9c10bfec791acfb2",
    '15m': "md5:34a416e7e25235d50ba609d2e94e8a49",
    'var': "md5:81fde79fabb87cf0941c875b6e23c03d"
}

fref = dict()
print('Using reference files:')
for ftype in OUTTYPES:
    fname = f'{Args.basename}_{ftype}.csv'
    ftmp = pooch.retrieve(
        f"http://nartex.fis.ucm.es/~ncl/rgbphot/gaia/{fname}",
        known_hash=auxhash[ftype]
    )
    fref[ftype] = ftmp
    print(ftmp)


def test_pleiades_simple():
    exec_rgblues(Args)
    for ftype in OUTTYPES:
        fname = f'{Args.basename}_{ftype}.csv'
        print(f'Checking {fname}')
        with open(fname, 'r') as f:
            file1 = f.read()
        with open(fref[ftype], 'r') as f:
            file2 = f.read()
        assert file1 == file2



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Compute RGB magnitudes
"""

import sys

from astropy import units as u
from astropy.table import Column
import numpy as np
from numpy.polynomial import Polynomial


def step6(r_edr3, verbose):
    """Cross-matching between DR2 and EDR3 queries

    Parameters
    ----------
    r_edr3 : astropy Table
        Table containing the EDR3 query.
    verbose : bool
        If True, display additional information.

    Returns
    -------
    r_edr3 : astropy Table
        Updated EDR3 table including the predicted RGB magnitudes.

    """
    sys.stdout.write('<STEP6> Computing RGB magnitudes...')
    sys.stdout.flush()
    # predict RGB magnitudes
    coef_B = np.array([-0.13748689, 0.44265552, 0.37878846, -0.14923841, 0.09172474, -0.02594726])
    coef_G = np.array([-0.02330159, 0.12884074, 0.22149167, -0.1455048, 0.10635149, -0.0236399])
    coef_R = np.array([0.10979647, -0.14579334, 0.10747392, -0.1063592, 0.08494556, -0.01368962])
    coef_X = np.array([-0.01252185, 0.13983574, 0.23688188, -0.10175532, 0.07401939, -0.0182115])

    poly_B = Polynomial(coef_B)
    poly_G = Polynomial(coef_G)
    poly_R = Polynomial(coef_R)
    poly_X = Polynomial(coef_X)

    r_edr3.add_column(
        Column(np.round(r_edr3['phot_g_mean_mag'] + poly_B(r_edr3['bp_rp']), 2),
               name='b_rgb', unit=u.mag, format='.2f'), index=3
    )
    r_edr3.add_column(
        Column(np.round(r_edr3['phot_g_mean_mag'] + poly_G(r_edr3['bp_rp']), 2),
               name='g_rgb', unit=u.mag, format='.2f'), index=4
    )
    r_edr3.add_column(
        Column(np.round(r_edr3['phot_g_mean_mag'] + poly_R(r_edr3['bp_rp']), 2),
               name='r_rgb', unit=u.mag, format='.2f'), index=5
    )
    r_edr3.add_column(
        Column(np.round(r_edr3['phot_g_mean_mag'] + poly_X(r_edr3['bp_rp']), 2),
               name='g_br_rgb', unit=u.mag, format='.2f'), index=6
    )
    print('OK')
    if verbose:
        r_edr3.pprint(max_width=1000)

    return r_edr3



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
RGB predictions of Gaia EDR3 stars

This code is hosted at https://github.com/guaix-ucm/rgblues
Authors: Nicolás Cardiel <cardiel@ucm.es>
         Sergio Pascual <sergiopr@fis.ucm.es>
         Rafael González <rafael08@ucm.es>

Usage example:
$ rgblues 56.66 24.10 1.0 12
"""

import argparse
import sys

from astropy.io import fits
import pooch

from .core.step1 import step1
from .core.step2 import step2
from .core.step3 import step3
from .core.step4 import step4
from .core.step5 import step5
from .core.step6 import step6
from .core.step7 import step7
from .gui.step8 import step8
from .version import version

MAX_SEARCH_RADIUS = 30  # degrees
EDR3_SOURCE_ID_15M_ALLSKY = 'edr3_source_id_15M_allsky.fits'
RGB_FROM_GAIA_ALLSKY = 'rgb_from_gaia_allsky.fits'


def right_ascension(ra_str):
    ra = float(ra_str)
    if not (0 <= ra <= 360):
        print('Right ascension must be 0 <= ra <= 360 degree')
        msg = f'Right ascension {ra} degree out of range'
        print(msg)
        raise ValueError(msg)
    return ra


def declination(dec_str):
    dec = float(dec_str)
    if not (-90 <= dec <= 90):
        print('Declination must be -90 <= dec <= 360 degree')
        msg = f'Declination {dec} degree out of range'
        print(msg)
        raise ValueError(msg)
    return dec


def search_radius(r_str):
    r = float(r_str)
    if not (0 < r < MAX_SEARCH_RADIUS):
        print(f'Search radius must be 0 < r <= {MAX_SEARCH_RADIUS} degree')
        msg = f'Search radius {r} degree out of range'
        print(msg)
        raise ValueError(msg)
    return r


def exec_rgblues(args):
    """Callable function that executes all the required steps

    Parameters
    ----------
    args: argparse instance
        Argparse instance containing the command-line parameters

    """
    print(f'\n        Welcome to rgblues version {version}')
    print(f'        ==============================\n')

    # check whether the auxiliary FITS binary table exists
    if args.debug:
        auxbintable = RGB_FROM_GAIA_ALLSKY
        auxhash = "md5:5e42a58471c780ef622d7bd620af3ea2"
    else:
        auxbintable = EDR3_SOURCE_ID_15M_ALLSKY
        auxhash = "md5:927f0dc8e74562268ee98bb13f6f88e3"

    fauxbin = pooch.retrieve(
        f"http://nartex.fis.ucm.es/~ncl/rgbphot/gaia/{auxbintable}",
        known_hash=auxhash
    )

    # read the previous file
    try:
        with fits.open(fauxbin) as hdul_table:
            edr3_source_id_15M_allsky = hdul_table[1].data.source_id
            if args.debug:
                table15M = hdul_table[1].data.copy()
            else:
                table15M = None
    except FileNotFoundError:
        raise SystemExit(f'ERROR: unexpected problem while reading {EDR3_SOURCE_ID_15M_ALLSKY}')

    # ---
    # step 1: EDR3 query
    r_edr3, nstars, nstars_colorcut = step1(
        args.ra_center,
        args.dec_center,
        args.search_radius,
        args.g_limit,
        args.verbose
    )

    # ---
    # step 2: retrieve additional parameters from the StarHorse sample
    if args.starhorse_block > 0:
        r_edr3 = step2(
            r_edr3,
            args.starhorse_block,
            args.verbose,
            args.debug
        )
    else:
        print('<STEP2> Retrieving StarHorse data from Gaia@AIP... (skipped!)')

    # ---
    # step 3: intersection with 15M star sample
    intersection = step3(
        r_edr3,
        edr3_source_id_15M_allsky,
        args.debug
    )

    # ---
    # step 4: DR2 query to identify variable stars
    r_dr2, nvariables, mask_var = step4(
        args.ra_center,
        args.dec_center,
        args.search_radius,
        args.g_limit,
        args.verbose
    )
    # ---

    # step 5: cross-match between DR2 and EDR3 to identify the variable stars
    if nvariables > 0:
        r_cross_var, nvariables = step5(
            r_dr2,
            mask_var,
            r_edr3,
            args.verbose
        )
    else:
        r_cross_var = None
        nvariables = 0
    print(f'        --> {nvariables} variable(s) in selected EDR3 star sample')

    # ---
    # step 6: compute RGB magnitudes
    r_edr3 = step6(
        r_edr3,
        args.verbose
    )

    # ---
    # step 7: generate output CSV files
    step7(
        r_edr3,
        args.basename,
        args.starhorse_block,
        nvariables,
        r_cross_var,
        intersection,
        table15M,
        args.verbose,
        args.debug
    )

    # ---
    # step 8: generate PDF chart
    if args.noplot:
        sys.stdout.write('<STEP8> No PDF plot generated (skipped!)\n')
        sys.stdout.flush()
    else:
        step8(
            r_edr3,
            args.ra_center,
            args.dec_center,
            args.search_radius,
            args.symbsize,
            args.brightlimit,
            args.nonumbers,
            args.nocolor,
            args.basename,
            nstars_colorcut,
            nvariables,
            r_cross_var,
            intersection,
            version,
            args.verbose
        )
    print('End of program')


def main():
    """Main function to parse input arguments"""
    parser = argparse.ArgumentParser(description=f"RGB predictions for Gaia EDR3 stars (version {version})")
    parser.add_argument("ra_center", help="right Ascension (decimal degrees)", type=right_ascension)
    parser.add_argument("dec_center", help="declination (decimal degrees)", type=declination)
    parser.add_argument("search_radius", help="search radius (decimal degrees)", type=search_radius)
    parser.add_argument("g_limit", help="limiting Gaia G magnitude", type=float)
    parser.add_argument("--basename", help="file basename for output files", type=str, default="rgblues")
    parser.add_argument("--brightlimit",
                        help="stars brighter than this Gaia G limit are displayed with star symbols (default=8.0)",
                        type=float, default=8.0)
    parser.add_argument("--symbsize", help="multiplying factor for symbol size (default=1.0)",
                        type=float, default=1.0)
    parser.add_argument("--nonumbers", help="do not display star numbers in PDF chart", action="store_true")
    parser.add_argument("--noplot", help="skip PDF chart generation", action="store_true")
    parser.add_argument("--nocolor", help="do not use colors in PDF chart", action="store_true")
    parser.add_argument("--starhorse_block", help="number of stars/query (default=0, no query)",
                        default=0, type=int)
    parser.add_argument("--verbose", help="increase program verbosity", action="store_true")
    parser.add_argument("--debug", help="debug flag", action="store_true")

    args = parser.parse_args()

    if len(sys.argv) == 1:
        parser.print_usage()
        raise SystemExit()

    exec_rgblues(args)


if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Generate output PDF plot
"""

import sys

import matplotlib.pyplot as plt
import matplotlib.style
from astropy import units as u
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
import numpy as np

from .style import mpl_style
from rgblues.core.step7 import OUTTYPES
OUTTYPES_COLOR = {'edr3': 'black', '15m': 'red', 'var': 'blue'}


def step8(r_edr3, ra_center, dec_center, search_radius, symbsize, brightlimit,
          nonumbers, nocolor, basename, nstars_colorcut, nvariables,
          r_cross_var, intersection, version, verbose):
    """Perform EDR3 query

    Parameters
    ----------
    r_edr3 : astropy Table
        Table containing the EDR3 query.
    ra_center : float
        Right ascension (decimal degree) corresponding to the center
        of the field of view.
    dec_center : float
        Declination (decimal degree) corresponding to the center
        of the field of view.
    search_radius : float
        Radius (decimal degrees) of the field of view.
    symbsize : float
        Multiplying factor for symbol size.
    brightlimit : float
        Stars brighter than this Gaia G limit are displayed with star
        symbols.
    nonumbers : bool
        If True, do not display star numbers in PDF chart.
    nocolor : bool
        If True, do not use colors in PDF chart.
    basename : str
        Base name for output files.
    nstars_colorcut : int
        Number of stars outside the colour cut
        -0.5 < G_BP - G_RP < 2.0 mag.
    nvariables : int
        Final number of variable stars in EDR3 query.
    r_cross_var : astropy Table
        Table with the cross-match of the DR2 and EDR3 queries.
    intersection : numpy array
        Array with the source_id values of the intersection between
        the EDR3 query and the 15M star sample.
    version : str
        Version number.
    verbose : bool
        If True, display additional information.

    """
    sys.stdout.write('<STEP8> Generating PDF plot...')
    sys.stdout.flush()

    # define WCS
    naxis1 = 1024
    naxis2 = naxis1
    pixscale = 2 * search_radius / naxis1

    wcs_image = WCS(naxis=2)
    wcs_image.wcs.crpix = [naxis1 / 2, naxis2 / 2]
    wcs_image.wcs.crval = [ra_center, dec_center]
    wcs_image.wcs.cunit = ["deg", "deg"]
    wcs_image.wcs.ctype = ["RA---TAN", "DEC--TAN"]
    wcs_image.wcs.cdelt = [-pixscale, pixscale]
    wcs_image.array_shape = [naxis1, naxis2]
    if verbose:
        print(wcs_image)

    # generate plot
    r_edr3.sort('phot_g_mean_mag')
    if verbose:
        print('')
        r_edr3.pprint(max_width=1000)

    symbol_size = symbsize * (50 / np.array(r_edr3['phot_g_mean_mag'])) ** 2.5
    ra_array = np.array(r_edr3['ra'])
    dec_array = np.array(r_edr3['dec'])

    c = SkyCoord(ra=ra_array * u.degree, dec=dec_array * u.degree, frame='icrs')
    x_pix, y_pix = wcs_image.world_to_pixel(c)

    matplotlib.style.use(mpl_style)
    fig = plt.figure(figsize=(13, 10))
    ax = plt.subplot(projection=wcs_image)
    iok = r_edr3['phot_g_mean_mag'] < brightlimit
    if nocolor:
        sc = ax.scatter(x_pix[iok], y_pix[iok], marker='*', color='grey',
                        edgecolors='black', linewidth=0.2, s=symbol_size[iok])
        ax.scatter(x_pix[~iok], y_pix[~iok], marker='.', color='grey',
                   edgecolors='black', linewidth=0.2, s=symbol_size[~iok])
    else:
        cmap = plt.cm.get_cmap('jet')
        sc = ax.scatter(x_pix[iok], y_pix[iok], marker='*',
                        edgecolors='black', linewidth=0.2, s=symbol_size[iok],
                        cmap=cmap, c=r_edr3[iok]['bp_rp'], vmin=-0.5, vmax=2.0)
        ax.scatter(x_pix[~iok], y_pix[~iok], marker='.',
                   edgecolors='black', linewidth=0.2, s=symbol_size[~iok],
                   cmap=cmap, c=r_edr3[~iok]['bp_rp'], vmin=-0.5, vmax=2.0)

    # display numbers if requested
    if not nonumbers:
        for irow in range(len(r_edr3)):
            number_csv = r_edr3[irow]['number_csv']
            text = r_edr3[irow][f'number_{OUTTYPES[number_csv]}']
            ax.text(x_pix[irow], y_pix[irow], text,
                    color=OUTTYPES_COLOR[OUTTYPES[number_csv]], fontsize='5',
                    horizontalalignment='left', verticalalignment='bottom')

    # stars outside the -0.5 < G_BP - G_RP < 2.0 colour cut
    if nstars_colorcut > 0:
        mask_colour = np.logical_or((r_edr3['bp_rp'] <= -0.5), (r_edr3['bp_rp'] >= 2.0))
        iok = np.argwhere(mask_colour)
        ax.scatter(x_pix[iok], y_pix[iok], s=240, marker='D', facecolors='none', edgecolors='grey', linewidth=0.5)

    # variable stars
    if nvariables > 0:
        sorter = np.argsort(r_edr3['source_id'])
        iok = np.array(sorter[np.searchsorted(r_edr3['source_id'], r_cross_var['dr3_source_id'], sorter=sorter)])
        ax.scatter(x_pix[iok], y_pix[iok], s=240, marker='s', facecolors='none', edgecolors='blue', linewidth=0.5)

    # stars in 15M sample
    if len(intersection) > 0:
        sorter = np.argsort(r_edr3['source_id'])
        iok = np.array(sorter[np.searchsorted(r_edr3['source_id'], np.array(list(intersection)), sorter=sorter)])
        ax.scatter(x_pix[iok], y_pix[iok], s=240, marker='o', facecolors='none',
                   edgecolors=OUTTYPES_COLOR['15m'], linewidth=0.5)

    ax.scatter(0.03, 0.96, s=240, marker='o', facecolors='white',
               edgecolors=OUTTYPES_COLOR['15m'], linewidth=0.5,
               transform=ax.transAxes)
    ax.text(0.06, 0.96, 'star in 15M sample', fontsize=12, backgroundcolor='white',
            horizontalalignment='left', verticalalignment='center', transform=ax.transAxes)

    ax.scatter(0.03, 0.92, s=240, marker='s', facecolors='white',
               edgecolors=OUTTYPES_COLOR['var'], linewidth=0.5,
               transform=ax.transAxes)
    ax.text(0.06, 0.92, 'variable in Gaia DR2', fontsize=12, backgroundcolor='white',
            horizontalalignment='left', verticalalignment='center', transform=ax.transAxes)

    ax.scatter(0.03, 0.88, s=240, marker='D', facecolors='white', edgecolors='grey', linewidth=0.5,
               transform=ax.transAxes)
    ax.text(0.06, 0.88, 'outside colour range', fontsize=12, backgroundcolor='white',
            horizontalalignment='left', verticalalignment='center', transform=ax.transAxes)

    ax.set_xlabel('ra')
    ax.set_ylabel('dec')

    ax.set_aspect('equal')

    if not nocolor:
        cbaxes = fig.add_axes([0.683, 0.81, 0.15, 0.02])
        cbar = plt.colorbar(sc, cax=cbaxes, orientation='horizontal', format='%1.0f')
        cbar.ax.tick_params(labelsize=12)
        cbar.set_label(label=r'$G_{\rm BP}-G_{\rm RP}$', size=12, backgroundcolor='white')

    ax.text(0.98, 0.96, f'Field radius: {search_radius:.4f} degree', fontsize=12, backgroundcolor='white',
            horizontalalignment='right', verticalalignment='center', transform=ax.transAxes)
    ax.text(0.02, 0.06, r'$\alpha_{\rm center}$:', fontsize=12, backgroundcolor='white',
            horizontalalignment='left', verticalalignment='bottom', transform=ax.transAxes)
    ax.text(0.25, 0.06, f'{ra_center:.4f} degree', fontsize=12, backgroundcolor='white',
            horizontalalignment='right', verticalalignment='bottom', transform=ax.transAxes)
    ax.text(0.02, 0.02, r'$\delta_{\rm center}$:', fontsize=12, backgroundcolor='white',
            horizontalalignment='left', verticalalignment='bottom', transform=ax.transAxes)
    ax.text(0.25, 0.02, f'{dec_center:+.4f} degree', fontsize=12, backgroundcolor='white',
            horizontalalignment='right', verticalalignment='bottom', transform=ax.transAxes)
    ax.text(0.98, 0.02, f'rgblues, version {version}', fontsize=12, backgroundcolor='white',
            horizontalalignment='right', verticalalignment='bottom', transform=ax.transAxes)

    f = np.pi / 180
    xp = naxis1 / 2 + search_radius/pixscale * np.cos(np.arange(361)*f)
    yp = naxis2 / 2 + search_radius/pixscale * np.sin(np.arange(361)*f)
    ax.plot(xp, yp, '-', color='orange', linewidth=0.5, alpha=0.5)

    ax.set_xlim([-naxis1*0.12, naxis1*1.12])
    ax.set_ylim([-naxis2*0.05, naxis2*1.05])

    ax.set_axisbelow(True)
    overlay = ax.get_coords_overlay('icrs')
    overlay.grid(color='black', ls='dotted')

    plt.savefig(f'{basename}.pdf')
    plt.close(fig)
    if verbose:
        pass
    else:
        print('OK')



# -*- coding: utf-8 -*-
#
# Copyright 2021 Universidad Complutense de Madrid
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

"""
Cross-matching between DR2 and EDR3 to identify the variable stars
"""

import sys

from astroquery.gaia import Gaia


def step5(r_dr2, mask_var, r_edr3, verbose):
    """Cross-matching between DR2 and EDR3 queries

    Parameters
    ----------
    r_dr2 : astropy Table
        Table containing the DR2 query.
    mask_var : numpy array
        Array indicating whether the star is variable or not
        in the DR2 query.
    r_edr3 : astropy Table
        Table containing the EDR3 query.
    verbose : bool
        If True, display additional information.

    Returns
    -------
    r_cross_var : astropy Table
        Table with the cross-match of the DR2 and EDR3 queries.
    nvariables : int
        Final number of variable stars in EDR3 query.

    """
    # generate sequence of source_id of variable stars
    dumstr = ','.join([str(item) for item in r_dr2[mask_var]['source_id']])
    # cross-match
    query = f"""
    SELECT *
    FROM gaiaedr3.dr2_neighbourhood
    WHERE dr2_source_id IN ({dumstr})
    ORDER BY angular_distance
    """
    sys.stdout.write('<STEP5> Cross-matching variables in DR2 with stars in EDR3... (please wait)\n  ')
    sys.stdout.flush()
    job = Gaia.launch_job_async(query)
    r_cross_var = job.get_results()
    if verbose:
        r_cross_var.pprint(max_width=1000)
    nvariables = len(r_cross_var)
    if nvariables > 0:
        # check that the variables pass the same selection as the EDR3 stars
        # (this includes de colour cut)
        mask_var = []
        for item in r_cross_var['dr3_source_id']:
            if item in r_edr3['source_id']:
                mask_var.append(True)
            else:
                mask_var.append(False)
        r_cross_var = r_cross_var[mask_var]
        nvariables = len(r_cross_var)
        if verbose:
            r_cross_var.pprint(max_width=1000)
    else:
        r_cross_var = None

    return r_cross_var, nvariables
